<?php
namespace Drupal\editoria11y\Controller;

use Drupal;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\editoria11y\Dashboard;
use Drupal\user\Entity\User;
use Drupal\Core\Routing\RouteObjectInterface;

/**
 * Provides route responses for the Editoria11y module.
 */
class DashboardController extends ControllerBase
{
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
    protected $account;

  /**
   * The current database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
    protected $connection;

  /**
   * @var Dashboard
   */
    protected $dashboard;

  /**
   * @param $dashboard
   */
    public function __construct($dashboard)
    {
        $this->dashboard = $dashboard;
    }

  /**
   * {@inheritdoc}
   */
    public static function create($container)
    {
        return new static(
            $container->get('editoria11y.dashboard')
        );
    }

    /**
     * Throws error if sync is disabled.
     */
    private function sync_status() {
      $config = \Drupal::config('editoria11y.settings');
      // force standard bool
      $sync = $config->get('disable_sync');
      if (!!$sync) {
        $msg = t("Dashboard sync is disabled in Editoria11y configuration.");
        \Drupal::messenger()->addWarning($msg, $type = 'warning');
      }
    }


    
    /**
     * Get lang from URL query or Drupal native code.
     *
     * @return string
     */
    private function get_page_lang() {
      $language = Drupal::request()->query->get('lang');
      if (!is_null($language)) {
        $lang = Xss::filter($language);
      } else {
        $lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
      }
      return $lang;
    }
    
    /**
     * Builds list of links to show results for specific languages.
     *
     * @return string
     */
    private function get_language_filters($lang) {
      $languages = $this->dashboard->getLanguages();
      $links = '';
      if ($languages['count'] > 1) {
        $links .= "<nav aria-label='". t('Language') . "'>" . t('Language') . ": ";
        $active = array(
          // Increase target size
          'attributes' => array(
            'aria-current' => array(
              'page'
            ),
          ),
        );
        foreach($languages['languages'] as $id) {
          if ($id->page_language === $lang) {
            $links .= "<strong>";
            $linkString = Url::fromRoute('<current>', ['lang' => $id->page_language], $active);
            $linkToLang = Link::fromTextAndUrl($id->page_language, $linkString)->toString();
            $links .= $linkToLang . "</strong> | ";
          } else {
            $linkString = Url::fromRoute('<current>', ['lang' => $id->page_language]);
            $linkToLang = Link::fromTextAndUrl($id->page_language, $linkString)->toString();
            $links .= $linkToLang . " | ";
          }
        }
        if ($lang === "all") {
          $links .= "<strong>";
          $linkString = Url::fromRoute('<current>', ['lang' => 'all'], $active); 
          $linkToLang = Link::fromTextAndUrl('all', $linkString)->toString(); 
          $links .= $linkToLang . "</strong>";
        } else {
          $linkString = Url::fromRoute('<current>', ['lang' => 'all']);
          $linkToLang = Link::fromTextAndUrl('all', $linkString)->toString(); 
          $links .= $linkToLang;
        }
      }
      $links .= "</nav>";
      return $links;
    }

    
    /**
     * Gets list of pages with issues
     *
     * @return array
     */
    public function get_test_results()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        // load filters
        //$form_state = new FormState();
        //$form_state->setRebuild();

        //$render['form'] = \Drupal::formBuilder()->buildForm('Drupal\editoria11y\Form\DashboardFilters', $form_state);
        //$test = $form_state->getValue('test');

        $header = [
        ['data' => t("Page"), 'field' => 't.page_path'],
        ['data' => t("Issues Found"), 'field' => 't.page_result_count', 'sort' => 'desc'],
        ['data' => t("Type"), 'field' => 't.entity_type'],
        ];
        $rows = [];

        // Filter results to active language and create filter links
        $lang = $this->get_page_lang();
        $links = $this->get_language_filters($lang);

        $results = $this->dashboard->getResults($page, $header, 50, $lang);
        foreach ($results["results"] as $record) {
          // to review: https://drupal.stackexchange.com/questions/144992/how-do-i-create-a-link
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $linkToPage = $record->page_title;
            }
            $linkString = Url::fromUserInput("/admin/reports/editoria11y/page?q=" . $record->page_path);
            $linkToIssueList = Link::fromTextAndUrl($record->page_result_count, $linkString)->toString();
            $rows[] = [$linkToPage, $linkToIssueList, $record->entity_type];
          // Todo add delete trashcan?
        }

        $render = [];        
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#prefix' => $links,
        '#cache' => [
          'contexts' => [
            'user',
            'url',
          ],
          'tags' => [
            'editoria11y.dashboard',
          ],
        ]
        ];
        $pager_manager
        ->createPager($results["count"], 50);
        $render[] = [
        '#type' => 'pager',
        ];


        return $render;
    }

    /**
     * Given a query variable of a page id, gets issues on page.
     *
     * @return array
     */
    public function get_issues_by_page()
    {
        $header = [
          ['data' => t("Count"), 'field' => 't.result_name_count'],
          ['data' => t("Issue"), 'field' => 't.result_name', 'sort' => 'asc'],
        ];

        $rows = [];

        $key = Drupal::request()->query->get('q');
        if (!is_null($key)) {
          $key = Xss::filter($key);
        }
        $results = $this->dashboard->getResultsByPage($key, $header);
        $page = "";

        foreach ($results as $record) {
            $page_url = Url::fromUserInput($record->page_path);
            $page = Link::fromTextAndUrl($record->page_title, $page_url)->toString();
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . $record->result_name);
            $pages_by_issue = Link::fromTextAndUrl($record->result_name, $pages_by_issue_url)->toString();
            $rows[] = [$record->result_name_count, $pages_by_issue];
        }

        return [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2>".t('Page: ') . $page ."</h2>",
        ]
        ],
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        ];
    }

    /**
     * Given a query variable of an issue name, builds table of issues.
     *
     * @return array
     */
    public function get_pages_by_issue()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        $header = [
          ['data' => t("Count"), 'field' => 'page_result_count', 'sort'=>'desc'],
          ['data' => t("Page"), 'field'=>'page_path'],
        ];

        $rows = [];
        $key = Drupal::request()->query->get('q');
        $results = $this->dashboard->getResultsByIssue($key, $page, $header);

        foreach ($results["results"] as $record) {
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $rows[] = [$record->result_name_count, $link];
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render[] = [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2>" . t('Pages with "' . $key . '" errors') . "</h2>",
        ]
        ],
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

    private function get_top_pages()
    {
        $header = [
          t("Page"),
          t("Count"),
        ];
        $rows = [];
        $lang = $this->get_page_lang();
        $results = $this->dashboard->getTopResults($lang);
        foreach ($results as $record) {
            if (strpos($record->page_path, "/") === 0) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url);
            } else {
                $linkToPage = $record->page_title;
            }
            $link_options = array(
              // Increase target size
              'attributes' => array(
                'class' => array(
                  'action-link--small'
                ),
              ),
            );
            $linkString = Url::fromRoute('editoria11y.reports_issues_by_page', ['q' => $record->page_path], $link_options);
            $linkToIssueList = Link::fromTextAndUrl($record->page_result_count, $linkString)->toString();
            $rows[] = [ $linkToPage, $linkToIssueList];
          // Todo add delete trashcan?
        }

        $render = [];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
          '#prefix' => '<h2>' . t("Pages with the most issues") .'</h2>',
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => t('View all pages with issues'),
          '#attributes' => ['class' => 'button button--primary'],
          '#url' => Url::fromRoute('editoria11y.reports_pages'),
          '#suffix' => '<br>&nbsp;</p>',
        ],
        ];

        $render[] = [
          'spacer' => [
            '#type' => 'html_tag',
            '#tag' => 'p',
          ],
        ];

        return $render;
    }

    /**
     * Gets summary info on recent dismissals for dashboard panel.
     *
     * @return array
     */
    private function get_recent_dismissals()
    {

        $result_service = \Drupal::service('editoria11y.dashboard');
        $lang = $this->get_page_lang();
        $results = $result_service->getRecentDismissals($lang);

        //   ['page_title','route_name','page_path','created','updated','result_name','dismissal_status','uid','created','updated', 'stale']);
        $header = [
          t('Page'),
          t('Issue'),
          t('Marked as'),
          t('User'),
          t('Date'),
        ];

        $rows = [];

        foreach ($results as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $date = \Drupal::service('date.formatter')->format($record->created);

          // todo: return username instead of user id
          // todo: show stale as better than numerical boolean
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, $name, $date];
        }

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#prefix' => '<h2>' . t("Recent dismissals") . '</h2>',
          '#header' => $header,
          '#rows' => $rows,
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => t('View all dismissals'),
          '#attributes' => ['class' => 'button button--primary'],
          '#url' => Url::fromRoute('editoria11y.reports_dismissals'),
          '#suffix' => '<br>&nbsp;</p>',
        ],
        ];
        return $render;
    }


    /**
     * Builds table of top issues. Count limiter used for dashboard summary panel.
     *
     * @return array
     */
    private function get_top_issues($count) {
        $result_service = \Drupal::service('editoria11y.dashboard');
        
        $language = Drupal::request()->query->get('lang');
        if (!is_null($language)) {
          $lang = Xss::filter($language);
        } else {
          $lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
        }
        $links = $this->get_language_filters($lang);
        
        $results = $result_service->getIssueList($count, $lang);
        
        $header = [
        t('Issue'),
        t('Count'),
        ];

        $rows = [];

        foreach ($results as $record) {
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . t($record->result_name));
            $pages_by_issue = Link::fromTextAndUrl($record->result_name, $pages_by_issue_url)->toString();
            $rows[] = [$pages_by_issue, $record->result_name_count];
        }

        $prefix = "";
        if ($count === 0 || $count > 25) {
          // Add language filters to page
          $prefix = $links;
        }
        $prefix .= "<h2>" . t("Most frequent issues") . "</h2>";

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
          '#prefix' => $prefix,
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        if ($count > 0) {
          $render[] = [
            'link' => [
              '#prefix' => '<p>',
              '#type' => 'link',
              '#title' => t('View all issues'),
              '#attributes' => ['class' => 'button button--primary'],
              '#url' => Url::fromRoute('editoria11y.reports_all_issues'),
              '#suffix' => '<br>&nbsp;</p>',
            ],
            ];
        }
        return $render;
    }


    /**
     * Builds page with all dismissals.
     *
     * @return array
     */
    public function get_dismissals()
    {

      // Todo: trashcans on dashboard

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        $header = [
          'page' => t("Page with issue"),
          'issue' => t("Dismissed issue"),
          'dismissals' => t("Marked as"),
          'user' => t("By user"),
          'stale' => t('Still on page'),
        ];

        $rows = [];

        $results = $this->dashboard->getDismissals($page);

        foreach ($results["results"] as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }

            $stale = $record -> stale ? "No" : "Yes";
            
            /* todo: remove record action
            $link_options = array(
              'attributes' => array(
                'class' => array(
                  'action-link action-link--icon-trash'
                ),
                'title' => $record->page_title,
              ),
            );

            $resetUrl = Url::fromRoute('editoria11y.reports_dismissals', ['rmpage'=>$record->page_path], $link_options);
            $resetLink = Link::fromTextAndUrl("Reset ", $resetUrl)->toString();
            */

          // todo: return username instead of user id
          // todo: show stale as better than numerical boolean
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, $name, $stale];
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
        'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

  /**
   * Page: summary dashboard with three panels.
   *
   * @return array
   *   A simple renderable array.
   */
    public function dashboard(): array
    {
      $this->sync_status();
      $language = Drupal::request()->query->get('lang');
      if (!is_null($language)) {
        $lang = Xss::filter($language);
      } else {
        $lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
      }
      $links = $this->get_language_filters($lang);
      

      $render = [
        '#type' => 'container',
        '#attributes' => array(
          'class' => 'layout-row',
        ),
        '#prefix' => $links,
      ];
      $render[] = [
        '#type' => 'container',
        '#attributes' => array(
          'class' => 'layout-column layout-column--half',
        ),
        'content' => [
          $this->get_top_pages(),

        ],
      ];
      $render[] = [
        '#type' => 'container',
        '#attributes' => array(
          'class' => 'layout-column layout-column--half',
        ),
        'content' => [
          $this->get_top_issues(5),
        ],
      ];
      $render[] = [
        '#type' => 'container',
        '#attributes' => array(
          'class' => 'layout-column',
        ),
        'content' => [
          $this->get_recent_dismissals(),
        ],
      ];
      return $render;
    }

  /**
   * Page: pages with issues.
   *
   * @return array
   *   A simple renderable array.
   */
  public function results(): array
  {
    $this->sync_status();  
    return $this->get_test_results();
  }

  /**
   * Page: list of issues by type.
   *
   * @return array
   *   A simple renderable array.
   */
  public function all_issues(): array
  {
    $this->sync_status();
    return $this->get_top_issues(0);
  }

  /**
   * Page: pages with a specific issue.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function pages_by_issue(): array
    {
      $this->sync_status();
      return $this->get_pages_by_issue();
    }

  /**
   * Page: issues on a given page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function issues_by_page(): array
    {
      $this->sync_status();
      return $this->get_issues_by_page();
    }

  /**
   * Page: dismissals.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, filter by user
    public function dismissals(): array
    {
      $this->sync_status();
      return $this->get_dismissals();
    }
}
