<?php

namespace Drupal\editoria11y;

use Drupal\Core\Database\Connection;
use Drupal\Core\Database\StatementInterface;

/**
 * Handles database calls for DashboardController.
 */
class Dashboard
{

    private $database;

    public function __construct(Connection $database)
    {
        $this->database = $database;
    }


    /**
    * Returns list of content languages present in results
    * @return array
    */
    public function getLanguages(): array {
      $langs = $this->database->select('editoria11y_results')
      ->fields('editoria11y_results', ['page_language'])
      ->groupBy('page_language');
      $count = $langs->countQuery()->execute()->fetchField();
      $languages = $langs->execute();
      return ["languages" => $languages, "count" => $count];
    }
    
    /**
     * Returns a sortable list of alerts and a count of the total.
     * @param int $page
     * @param string $header
     *
     * @return array
     */
    public function getResults(int $page = 0, $header = ['page_title'], $range = 50, $language = 'all'): array
    {
        $num_per_page = $range;
        $offset = $num_per_page * $page;

        $query = $this->database->select('editoria11y_results', 't')
        ->extend('Drupal\\Core\\Database\\Query\\TableSortExtender');
        $query->fields('t', ['page_path','page_title','page_result_count','entity_type','page_language'])
        // We have to group by all fields due to sql_mode=only_full_group_by:
        // "SELECT list is not in GROUP BY clause and contains nonaggregated column"
        ->groupBy('page_path')
        ->groupBy('page_title')
        ->groupBy('entity_type')
        ->groupBy('page_language')
        ->groupBy('page_result_count');
        if ($language !== 'all') {
          $query->condition('page_language', $language);
        }
        $query
        ->orderByHeader($header)
        ->range($offset, $offset + $num_per_page); // Add table sort extender.
        $total = $this->database->select('editoria11y_results')
        ->fields('editoria11y_results', ['page_path'])
        ->groupBy('page_path');
        $count = $total->countQuery()->execute()->fetchField();
        
        

        $results = $query
        ->execute();

        return ["results" => $results, "count" => $count];
    }


    public function getTopResults($language = 'all'): StatementInterface {

        $query = $this->database->select('editoria11y_results');
        $query->fields('editoria11y_results', ['page_result_count', 'page_path', 'page_title', 'page_language'])
        ->groupBy('page_path')
        ->groupBy('page_title')
        ->groupBy('page_language')
        ->groupBy('page_result_count');
        if ($language !== 'all') {
          $query->condition('page_language', $language);
        }
        $query
        ->orderBy('page_result_count', 'desc')
        ->orderBy('page_title', 'asc')
        ->range(0, 15);

        return $query->execute();
    }

  /**
   * Gets pages with a given issue.
   * @return array
   */
    public function getResultsByIssue($resultKey, $page, $header): array
    {

        $num_per_page = 50;
        $offset = $num_per_page * $page;

        $query = $this->database->select('editoria11y_results', 't')
        ->extend('Drupal\\Core\\Database\\Query\\TableSortExtender');
        $query->fields('t')
        //->fields('editoria11y_results',['page_title','route_name','page_path','result_name_count'])
        ->condition('result_name', $resultKey)
        ->orderByHeader($header);

        $count = $query->countQuery()->execute()->fetchField();
        $results = $query->range($offset, $offset + $num_per_page)->execute();

        return ["results" => $results, "count" => $count];
    }

  /**
   * Given a path, returns issues found on page.
   * @return \Drupal\Core\Database\StatementInterface|null
   */
    public function getResultsByPage($resultKey, $header): ?StatementInterface
    {
        $query = $this->database->select('editoria11y_results', 't')
        ->extend('Drupal\\Core\\Database\\Query\\TableSortExtender');
        $query->fields('t')
        //->fields('editoria11y_results',['page_title','route_name','page_path','result_name_count'])
        ->condition('page_path', $resultKey)
        ->orderByHeader($header);

        return $query->execute();
    }

    /**
     * Gets types of issues found, and a count of number of pages with issue.
     * @return \Drupal\Core\Database\StatementInterface|null
     */
    public function getIssueList(int $count = 0, $language = 'all'): ?StatementInterface
    {
        $query = $this->database->select('editoria11y_results');
        $query->fields('editoria11y_results', ['result_name', 'page_language'])
        ->groupBy('result_name')
        ->groupBy('page_language')
        ->addExpression('sum(result_name_count)', 'result_name_count');
        if ($language !== 'all') {
          $query->condition('page_language', $language);
        }
        $query
        ->orderBy('result_name_count', 'desc')
        ->orderBy('result_name', 'asc');
        if ($count > 0) {
          $query->range(0, $count);
        }

        return $query->execute();
    }


  /**
   * Gets list of most recent "mark as OK" or "hide alert" actions.
   * @return \Drupal\Core\Database\StatementInterface
   */
  public function getRecentDismissals(): StatementInterface {
    $query = $this->database->select('editoria11y_dismissals')
      ->fields('editoria11y_dismissals', ['page_title','page_path','created','result_name','dismissal_status','uid'])
      ->condition('stale','0')
      ->orderBy('created', 'DESC');

    $results = $query->range(0, 5);

    return $results->execute();
  }

    /**
     * Gets all "mark as OK" or "hide alert" actions.
     * @return array
     */
    public function getDismissals($page): array
    {

        $num_per_page = 50;
        $offset = $num_per_page * $page;
        // Hardcoding stale exclusion until filters and deletions are available
        // todo: stale maintenance
        $query = $this->database->select('editoria11y_dismissals')
        ->fields('editoria11y_dismissals', ['page_title','route_name','page_path','created','updated','result_name','dismissal_status','uid','created','updated', 'stale']);

        $count = $query->countQuery()->execute()->fetchField();
        $results = $query->range($offset, $offset + $num_per_page)->execute();

        return ["results" => $results, "count" => $count];
    }
}
